// you MUST have these variables.
const ONKEYDOWN	= 0;
const ONKEYUP	= 1;
const ONKEYPRESS	= 2;

var KeyLog = {
	//KeyLog variables
	keyIDs: new Array(),
	//Action variables
	size: 0,
	actionKeys: new Array(),
	actionTriggers: new Array(),
	actionFunctions: new Array(),
	
	//==================================
	// KEY METHODS
	//==================================
	/*
	 * Pre-conditions:  A keyID is passed, ie KEY_ENTER, KEY_UP, KEY_A, etc.
	 * Post-conditions: If not already there, adds that keyID to the key array.
	 * 			  Does any existing ONKEYDOWN actions.
	 */
	addKey: function(keyID)
	{
		for(var i in this.keyIDs)
			if(this.keyIDs[i] == keyID)
				return;
		
		this.checkForTriggers(keyID,ONKEYDOWN);
		
		this.keyIDs.push(keyID);
	},
	/*
	 * Pre-conditions:  None.
	 * Post-conditions: Searches through all keys in the array.
	 *			  If there is an unpressed key, run ONKEYUP, if it has one. Delete that key from keyIDs.
	 */
	runKeyActions: function()
	{
		for(var i=0;i<this.keyIDs.length;i++)
		{
			if(!IsKeyPressed(this.keyIDs[i]))
			{
				this.checkForTriggers(this.keyIDs[i],ONKEYUP);
				
				if(this.hasBothKeys(this.keyIDs[i]))
					this.checkForTriggers(this.keyIDs[i],ONKEYPRESS);
				
				this.keyIDs.splice(i,1);
			}
		}
	},
	/*
	 * Pre-conditions:  keyID (KEY_ENTER, etc) passed.
	 * Post-conditions: returns true if the key has both ONKEYUP and ONKEYDOWN events, else false.
	 */
	hasBothKeys: function(keyID)
	{
		var hasKeys = 0;
		for(var i=0;i<this.size;i++)
		{
			if(keyID == this.actionKeys[i])
				if(this.actionTriggers[i] == ONKEYUP || this.actionTriggers[i] == ONKEYDOWN)
					hasKeys++;
		}
		return hasKeys == 2;
	},
	/*
	 * Pre-conditions:  keyID (KEY_ENTER, etc), and triggerType (ONKEYPRESS, etc) passed.
	 * Post-conditions: Searches through all keys and trigger types in the arrays.
	 *			  If there is a match, run the correct actionFunctions for it.
	 */
	checkForTriggers: function(keyID, triggerType)
	{
		for(var i=0;i<this.size;i++)
			if(keyID == this.actionKeys[i] && triggerType == this.actionTriggers[i])
				eval(this.actionFunctions[i]);
	},
	//==================================
	// ACTION METHODS
	//==================================
	/*
	 * Pre-conditions:  keyID, triggerType (ONKEYPRESS, etc), and function to use (in a string!) passed.
	 * Post-conditions: increase size var by 1, add keyID to the actionKeys array, add triggerType
	 * 			  to the actionTriggers array , and func to the actionFunctions array.
	 */
	addAction: function(keyID, triggerType, func)
	{
		this.size++;
		this.actionKeys.push(keyID);
		this.actionTriggers.push(triggerType);
		this.actionFunctions.push(func);
	},
	/*
	 * Pre-conditions:  keyID and triggerType passed.
	 * Post-conditions: Searches through all keys and trigger types in the arrays. If there is a match,
	 *			  decrease size var by 1, remove keyID from the actionKeys array,
	 * 			  and remove triggerType from the actionTriggers array.
	 */
	removeAction: function(keyID, triggerType)
	{
		for(var i=0;i<this.size;i++)
		{
			if(this.actionKeys[i] == keyID && this.actionTriggers[i] == triggerType)
			{
				this.size--;
				this.actionKeys.splice(i,1);
				this.actionTriggers.splice(i,1);
				this.actionFunctions.splice(i,1);
				
				break;
			}
		}
	}
};